# MOS reduction script - part 2 
# This part separates wavelengths, corrects for flatfield and wavelength-calibrates the science data
#----------------------------------------------------------------------------------------------------
# GG, Apr 2015
# GG, Jun 2018 Version for S16A-Q-17, which uses two central wavelengths only.
# GG, Jun3 2020: Moved the "Cutting spectra" section inside the wavelength loop in order to implelemt the refimage option in gstransform.


print "################################################################"
print "-------------------Building lists.....-----------------------"
string *list1
string *list2
delete.verify=no
delete nssci*.lst
delete flats*.lst
delete gsreduce_step2.log
unlearn gsreduce
gsreduce.logfile="gsreduce_step2.log"



# discriminate between central wavelengths
hselect ("gsrS*[0]","$I", "WAVELENG=5750", missing="INDEF", >& 'nssci_1red.lst')
hselect ("gsrS*[0]","$I", "WAVELENG=5800", missing="INDEF", >& 'nssci_2red.lst')
hselect ("gsrS*[0]","$I", "WAVELENG=5850", missing="INDEF", >& 'nssci_3red.lst')
hselect ("fS*[0]","$I", "WAVELENG=5750", missing="INDEF", >& 'flats_1.lst')
hselect ("fS*[0]","$I", "WAVELENG=5800", missing="INDEF", >& 'flats_2.lst')
hselect ("fS*[0]","$I", "WAVELENG=5850", missing="INDEF", >& 'flats_3.lst')


# get rid of ".fits[0]" in all lines in lists
! awk '{gsub(/.fits\[0\]/, "") ; print}' nssci_1red.lst > nssci_1red.txt
! awk '{gsub(/.fits\[0\]/, "") ; print}' nssci_2red.lst > nssci_2red.txt
! awk '{gsub(/.fits\[0\]/, "") ; print}' nssci_3red.lst > nssci_3red.txt

# List of flats, one line per wavelength - comment out those not being used otherwise se not being used otherwise you will get an error with gsreduce.
delete flats_red.txt
! awk 'NR==1 {gsub(/.fits\[0\]/, "") ; print}' flats_1.lst >> flats_red.txt
! awk 'NR==1 {gsub(/.fits\[0\]/, "") ; print}' flats_2.lst >> flats_red.txt
! awk 'NR==1 {gsub(/.fits\[0\]/, "") ; print}' flats_3.lst >> flats_red.txt



print "################################################################"
print "-------------------Starting Flatfielding-----------------------"
# Second gsreduce for science: flatfield correction and BPM


# This loops over wavelengths
list1 = 'flats_red.txt'
i=0
while (fscan (list1, s1) != EOF){
	i=i+1
	print(s1)
	gsreduce("@nssci_"//i//"red.txt",  flat=s1, logfile="gsreduce_step2.log", rawpath="./", fl_over-, fl_trim-, fl_bias-, fl_dark-, fl_flat+, fl_gmosaic-, fl_fixpix-, fl_gsappwave-, fl_cut-, ovs_flinter-, fl_vardq-)
}

print "######################################################################"
print "--------------------First Cosmic Rays rejection-----------------------"
delete tmp.txt
ls gsgsrS*.fits >> tmp.txt
list1 = 'tmp.txt'
# if using sky subtraction, the prefix is "c"
# if NOT using sky subtraction, the prefix is "nc"
while (fscan (list1, s1) != EOF){
gscrrej(s1, "nc"//s1, datares=4.0, fnsigma=8.0, niter=5, tnsigma=10.0, fl_inter-, logfile="gsreduce_step2.log")
}


print "################################################################"
print "------------------------ NO Sky subtraction-------------------------"
# gnsskysub cgsgsrS* outpref="n" fl_fixnc+ logfile="gsreduce_step2.log"


print "################################################################"
print "----------------------  Mosaicing data  ------------------------"
gmosaic ncgsgsrS*.fits fl_fixpix+ logfile='gmosaic_sci.log'




print "################################################################"
print "---------------  Wavelength calibrating frames -----------------"


# gstransformed arcs are 'tugpS'
! rm -f tarc_*.lst
hselect upS*[0] $I "WAVELENG=5750" >> tarc_1.lst
hselect upS*[0] $I "WAVELENG=5800" >> tarc_2.lst
hselect upS*[0] $I "WAVELENG=5850" >> tarc_3.lst

# List of arcs, one line per wavelength - comment out those not being used otherwise you will get an error with gstransform.
delete tarc*.txt

! awk 'NR==1 {gsub(/.fits\[0\]/, "") ; print}' tarc_1.lst >> tarc_1.txt
! awk 'NR==1 {gsub(/.fits\[0\]/, "") ; print}' tarc_2.lst >> tarc_2.txt
! awk 'NR==1 {gsub(/.fits\[0\]/, "") ; print}' tarc_3.lst >> tarc_3.txt

! awk 'NR==1 {gsub(/.fits\[0\]/, "") ; print}' tarc_1.lst >> tarc.txt
! awk 'NR==1 {gsub(/.fits\[0\]/, "") ; print}' tarc_2.lst >> tarc.txt
! awk 'NR==1 {gsub(/.fits\[0\]/, "") ; print}' tarc_3.lst >> tarc.txt


# discriminate between central wavelengths
! rm -f nssci_*skysub.lst
hselect mncgsgsrS*[0] $I "WAVELENG=5750" >> nssci_1skysub.lst
hselect mncgsgsrS*[0] $I "WAVELENG=5800" >> nssci_2skysub.lst
hselect mncgsgsrS*[0] $I "WAVELENG=5850" >> nssci_3skysub.lst

# get rid of ".fits[0]" in all lines in lists
! rm -f nssci_*skysub.txt
! awk '{gsub(/.fits\[0\]/, "") ; print}' nssci_1skysub.lst > nssci_1skysub.txt
! awk '{gsub(/.fits\[0\]/, "") ; print}' nssci_2skysub.lst > nssci_2skysub.txt
! awk '{gsub(/.fits\[0\]/, "") ; print}' nssci_3skysub.lst > nssci_3skysub.txt


delete mosaiced_sci.txt
ls mncgsgsrS*.fits >> mosaiced_sci.txt



# This loops over wavelengths
print "Transformation to wavelength"
list1 = 'tarc.txt'
i=0
j=0
while (fscan (list1, s1) != EOF){
	i=i+1
        print "---------------------  Cutting spectra ------------------------"
        # ADJUST THE 'yoffset' PARAMETER IF NEEDED WITHN GSCUT - INSTEAD OF GSCUT, USE GSREDUCE FOR CUTTING BECAUSE GSCUT DOEN NOT SUPPORT LISTS.
	    #gscut ("@nssci_"//i//"skysub.txt",outimage="um"//s1,logfile='gsreduce_step2.log', yoffset=82.,gradimage="", refimage=s1 )
        gsreduce ("@nssci_"//i//"skysub.txt",outpref="u",logfile='gsreduce_step3.log', yoffset=82., fl_cut+ , gradimage="", refimage=s1,fl_over-,fl_bias-,fl_flat- )
 
    print "---------------------  Transforming spectra to Wavelength ------------------------"
	gstransform("u@nssci_"//i//"skysub.txt", wavtraname=s1, logfile='gsreduce_step3.log')
}

! awk '{print "tu"($1)}' nssci_1skysub.txt > nssci_1skysubw.txt
! awk '{print "tu"($1)}' nssci_2skysub.txt > nssci_2skysubw.txt
! awk '{print "tu"($1)}' nssci_3skysub.txt > nssci_3skysubw.txt

print ""
print "Transformed"
print ""

print "################################################################"
print "---------------------  Combining frames ------------------------"
# Combining spectra - Assumes calibrated science (i.e. gstransform on science done)
# This basically is 'gemcombine' execpt for cases when there is only one frame - whence the nested loops.
# Modified Aug 2016, GG: it is better to gemcombine all frames together in order to remove CRs more efficiently
#    then I moved the gemcombine line out of the loops.


string *list1
string *list2
delete transformed_sci.txt

# This loops over wavelengths
print "Transformation to wavelength"
list1 = 'tarc.txt'
i=0
print (i)
while (fscan (list1, s1) != EOF){
	i=i+1
        print (i)
	print ""
	print (s1)
	j=0
	list2='nssci_'//(i)//'skysubw.txt'
	while (fscan (list2, s2) != EOF){
		j=j+1
	}

 	#special cases for low numbers of images
        if (j == 1) {
                printlog ("GEMCOMBINE: only one image to combine.", logfile="gemcombine_step2.log", verbose=yes) 
                printlog (s2, logfile="gemcombine_step2.log", verbose=yes)
		copy(s2//".fits", "sci"//i//"w.fits")
	} else {
	print list2
#aug2016	print "gemcombining"
#aug2016	gemcombine("@nssci_"//i//"skysubw.txt", output="sci"//i//"w.fits", logfile='gemcombine_step2.log')
}
}

#aug2016 ls sci*w.fits >> transformed_sci.txt

# New block
! awk '{print}' nssci_*skysubw.txt  >> tmp_all_sci_w.txt
print "gemcombining"
gemcombine("@tmp_all_sci_w.txt", output="sci_w.fits", combine='median', offsets='wcs', logfile='gemcombine_step2.log')


ls sci_w.fits >> transformed_sci.txt
gemextn @transformed_sci.txt


delete extlist.txt,extlist_mask.txt,extlist_res.txt


print "################################################################"
print "--------------  Further removing CR from spectra ---------------"
# Assumes calibrated science (i.e. gstransform on science done) and same wavelengths combined
# This is an adaptation of 'crutil' for MEF fits
  
string *list1
string *list2
crutil
#fitsutil

list1 = 'transformed_sci.txt'  

while (fscan (list1, s1) != EOF){

	delete phu.fits
	imcopy (s1//"[0]", "c"//s1//"[0]", verbose=yes)

delete ext_sci.txt
gemextn (s1, extname="SCI", extversion="1-", outfile="ext_sci.txt", logfile="gemextn.log")
i=1
list2 = 'ext_sci.txt' 
while (fscan (list2, s2) != EOF){

	print(i)
#default
	crutil.crmedian (s2, output="cleaned"//i//".fits")

#last used
#	crutil.crmedian (s2, output="cleaned"//i//".fits", median="", sigma="", var0=0., var1=0.,var2=0., lsigma=10., hsigma=3., ncmed=10, nlmed=15, ncsig=10, nlsig=10)
## older
#	crutil.crmedian (s2, output="cleaned"//i//".fits", median="", sigma="", var0=0., var1=0.,var2=0., lsigma=10., hsigma=9., ncmed=15, nlmed=25, ncsig=15, nlsig=25)
#	crutil.crmedian (s1//"[SCI,"//i//"]", output="cleaned"//i//".fits", crmask="mask_ext_"//i//".fits", median="", sigma="", residual="res_ext_"//i//".fits", var0=0., var1=0.,var2=0., lsigma=10., hsigma=9., ncmed=25, nlmed=25, ncsig=25, nlsig=25)

	print ("extension "//i," cleaned")
	dir ("cleaned"//i//".fits", >>& 'extlist.txt')
#	dir ("mask_ext_"//i//".fits", >>& 'extlist_mask.txt')
#	dir ("res_ext_"//i//".fits", >>& 'extlist_res.txt')
	print ""
	imreplace ("cleaned"//i//".fits", 0. ,upper=0.)
	imcopy ("cleaned"//i//".fits", "c"//s1//"[SCI,"//i//"]")
		i=i+1
        j=i
	print(j)
    }

	print "-------Rebuilding the MEF file------"

####====Inserting the MDF====####
imgets(s1//"[0]",'GEMCOMB')
print (imgets.value)
if (imgets.value == '0') {
	print("not gemcombined")
	fxinsert (s1//"["//1//"]", "c"//s1//"["//i-1//"]","0", verbose=no)
} else { 
	print("gemcombined")
	fxinsert (s1//"["//i//"]", "c"//s1//"["//i-1//"]","0", verbose=no)
}
####=========================####

	gemextn ("c"//s1)
	imdelete cleaned*.fits
	delete extlist.txt,extlist_mask.txt,extlist_res.txt


}


delete cr_cleaned_sci.txt,reduced_spectra.*
ls c*w.fits >> cr_cleaned_sci.txt
! awk '{print ($1)"[0]"}' cr_cleaned_sci.txt >> reduced_spectra.lst
hselect @reduced_spectra.lst $I,OBJECT,WAVELENG yes >> reduced_spectra.txt


print "################################################################"
print "      ---------Done--------         "
